# frozen_string_literal: true

#
# Copyright (C) 2011 Instructure, Inc.
#
# This file is part of Canvas.
#
# Canvas is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# Canvas is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
#

require_relative "../api_spec_helper"

describe "Services API", type: :request do
  before :once do
    user_with_pseudonym(active_all: true)
  end

  before do
    stub_kaltura
  end

  it "checks for auth" do
    get("/api/v1/services/kaltura")
    assert_status(401)
  end

  it "checks for auth on session" do
    post("/api/v1/services/kaltura_session")
    assert_status(401)
    expect(response.body).to include("must be logged in to use Kaltura")
  end

  it "returns the config information for kaltura" do
    json = api_call(:get, "/api/v1/services/kaltura",
                    controller: "services_api", action: "show_kaltura_config", format: "json")
    expect(json).to eq({
                         "enabled" => true,
                         "domain" => "kaltura.example.com",
                         "resource_domain" => "cdn.kaltura.example.com",
                         "rtmp_domain" => "rtmp.kaltura.example.com",
                         "partner_id" => "100",
                       })
  end

  it "degrades gracefully if kaltura is disabled or not configured" do
    allow(CanvasKaltura::ClientV3).to receive(:config).and_return(nil)
    json = api_call(:get, "/api/v1/services/kaltura",
                    controller: "services_api", action: "show_kaltura_config", format: "json")
    expect(json).to eq({
                         "enabled" => false,
                       })
  end

  it "returns a new kaltura session" do
    kal = double("CanvasKaltura::ClientV3")
    expect(kal).to receive(:startSession).and_return "new_session_id_here"
    allow(CanvasKaltura::ClientV3).to receive(:new).and_return(kal)
    json = api_call(:post, "/api/v1/services/kaltura_session",
                    controller: "services_api", action: "start_kaltura_session", format: "json")
    expect(json.delete_if { |k| %w[serverTime].include?(k) }).to eq({
                                                                      "ks" => "new_session_id_here",
                                                                      "subp_id" => "10000",
                                                                      "partner_id" => "100",
                                                                      "uid" => "#{@user.id}_#{Account.default.id}",
                                                                    })
  end

  it "returns a new kaltura session with upload config if param provided" do
    kal = double("CanvasKaltura::ClientV3")
    expect(kal).to receive(:startSession).and_return "new_session_id_here"
    allow(CanvasKaltura::ClientV3).to receive(:new).and_return(kal)
    json = api_call(:post, "/api/v1/services/kaltura_session",
                    controller: "services_api", action: "start_kaltura_session",
                    format: "json", include_upload_config: 1)
    expect(json.delete_if { |k| %w[serverTime].include?(k) }).to eq({
                                                                      "ks" => "new_session_id_here",
                                                                      "subp_id" => "10000",
                                                                      "partner_id" => "100",
                                                                      "uid" => "#{@user.id}_#{Account.default.id}",
                                                                      "kaltura_setting" => {
                                                                        "domain" => "kaltura.example.com",
                                                                        "kcw_ui_conf" => "1",
                                                                        "max_file_size_bytes" => 534_773_760,
                                                                        "partner_id" => "100",
                                                                        "player_ui_conf" => "1",
                                                                        "resource_domain" => "cdn.kaltura.example.com",
                                                                        "rtmp_domain" => "rtmp.kaltura.example.com",
                                                                        "subpartner_id" => "10000",
                                                                        "upload_ui_conf" => "1",
                                                                        "entryUrl" => "http://kaltura.example.com/index.php/partnerservices2/addEntry",
                                                                        "uiconfUrl" => "http://kaltura.example.com/index.php/partnerservices2/getuiconf",
                                                                        "uploadUrl" => "http://kaltura.example.com/index.php/partnerservices2/upload",
                                                                        "partner_data" => {
                                                                          "root_account_id" => @user.account.root_account.id,
                                                                          "sis_source_id" => nil,
                                                                          "sis_user_id" => nil
                                                                        },
                                                                      },
                                                                    })
  end
end
